package com.genuitec.sapphire.integratededitor.resource;

import static org.eclipse.sapphire.modeling.xml.XmlUtil.createQualifiedName;

import java.util.Set;

import javax.xml.namespace.QName;

import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.PossibleTypesService;
import org.eclipse.sapphire.Resource;
import org.eclipse.sapphire.modeling.xml.ChildXmlResource;
import org.eclipse.sapphire.modeling.xml.StandardXmlListBindingImpl;
import org.eclipse.sapphire.modeling.xml.XmlElement;
import org.eclipse.sapphire.modeling.xml.XmlNamespaceResolver;
import org.eclipse.sapphire.modeling.xml.XmlPath;
import org.eclipse.sapphire.modeling.xml.XmlResource;
import org.eclipse.sapphire.modeling.xml.annotations.XmlListBinding;

public class CustomizedXmlBindingImpl extends StandardXmlListBindingImpl {
	
	private XmlResource xmlResource;

	public CustomizedXmlBindingImpl(XmlResource xmlResource) {
		this.xmlResource = xmlResource;
	}
	
	protected void initBindingMetadata()
    {
        final XmlListBinding annotation = property().definition().getAnnotation( XmlListBinding.class );
        final XmlNamespaceResolver xmlNamespaceResolver = xmlResource.getXmlNamespaceResolver();
        
        final Set<ElementType> possible = property().service(PossibleTypesService.class).types();
        this.modelElementTypes = possible.toArray( new ElementType[ possible.size() ] );

        if( annotation == null )
        {
            this.path = new XmlPath( property().name(), xmlNamespaceResolver );
            
            this.xmlElementNames = new QName[ this.modelElementTypes.length ];
            
            for( int i = 0; i < this.modelElementTypes.length; i++ )
            {
                this.xmlElementNames[ i ] = createDefaultElementName( this.modelElementTypes[ i ], xmlNamespaceResolver );
            }
        }
        else
        {
            if( annotation.path().length() > 0 )
            {
                this.path = new XmlPath( annotation.path(), xmlNamespaceResolver );
            }
            
            final XmlListBinding.Mapping[] mappings = annotation.mappings();
            this.xmlElementNames = new QName[ this.modelElementTypes.length ];
            
            for( int i = 0; i < this.modelElementTypes.length; i++ )
            {
                final ElementType type = this.modelElementTypes[ i ];
                        
                for( XmlListBinding.Mapping mapping : mappings )
                {
                    if( mapping.type() == type.getModelElementClass() )
                    {
                        final String mappingElementName = mapping.element().trim();
                        
                        if( mappingElementName.length() == 0 )
                        {
                            throw new RuntimeException( "Element name must be specified in @XmlListBinding.Mapping annotation." );
                        }

                        this.xmlElementNames[ i ] = createQualifiedName( mappingElementName, xmlNamespaceResolver );
                        
                        break;
                    }
                }
                
                if( this.xmlElementNames[ i ] == null )
                {
                    this.xmlElementNames[ i ] = createDefaultElementName( type, xmlNamespaceResolver );
                }
            }
        }
    }
	 
	@Override
    protected Resource resource( final Object obj )
    {
        final XmlElement xmlElement = (XmlElement) obj;
        
        return new ChildXmlResource( xmlResource, xmlElement );
    }
 
    protected XmlElement getBaseXmlElement( final boolean createIfNecessary )
    {
        return xmlResource.getXmlElement( createIfNecessary );
    }

}
