/*
 * Copyright 2010, Genuitec, LLC
 * All Rights Reserved.
 */
package com.genuitec.sapphire.integratededitor.ui;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.Path;
import org.eclipse.sapphire.Element;
import org.eclipse.sapphire.ElementType;
import org.eclipse.sapphire.ListProperty;
import org.eclipse.sapphire.Property;
import org.eclipse.sapphire.PropertyBinding;
import org.eclipse.sapphire.PropertyDef;
import org.eclipse.sapphire.modeling.CorruptedResourceExceptionInterceptor;
import org.eclipse.sapphire.modeling.xml.RootXmlResource;
import org.eclipse.sapphire.modeling.xml.XmlResource;
import org.eclipse.sapphire.ui.CorruptedResourceExceptionInterceptorImpl;
import org.eclipse.sapphire.ui.SapphireEditor;
import org.eclipse.sapphire.ui.def.DefinitionLoader;
import org.eclipse.sapphire.ui.def.DefinitionLoader.Reference;
import org.eclipse.sapphire.ui.def.EditorPageDef;
import org.eclipse.sapphire.ui.forms.swt.MasterDetailsEditorPage;
import org.eclipse.sapphire.ui.swt.xml.editor.XmlEditorResourceStore;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.wst.sse.ui.StructuredTextEditor;

import com.genuitec.sapphire.integratededitor.annotations.IntegratedListBinding;
import com.genuitec.sapphire.integratededitor.model.IAndroidResources;
import com.genuitec.sapphire.integratededitor.model.IIntegratedAndroidResources;
import com.genuitec.sapphire.integratededitor.resource.CustomizedXmlBindingImpl;
import com.genuitec.sapphire.integratededitor.resource.RootIntegratedResource;

public final class AndroidResourcesEditor extends SapphireEditor {
	
	private ElementType applicationElementType = IAndroidResources.TYPE;
	private ElementType integratedAppElementType = IIntegratedAndroidResources.TYPE;
	
    private StructuredTextEditor colorsSourceEditor;
    private StructuredTextEditor stringsSourceEditor;
    
    private IAndroidResources androidResources;
    private IIntegratedAndroidResources integratedAndroidResources;
    
    private MasterDetailsEditorPage applicationDesignPage;
	private RootXmlResource foreignResource;
	

	@Override
	protected void createSourcePages() throws PartInitException {
		
        this.colorsSourceEditor = new StructuredTextEditor();
        this.colorsSourceEditor.setEditorPart(this);
        
        final FileEditorInput rootEditorInput = (FileEditorInput) getEditorInput();
        
        int index = addPage( this.colorsSourceEditor, rootEditorInput );
        setPageText( index, "Colors" );

        this.stringsSourceEditor = new StructuredTextEditor();
        this.stringsSourceEditor.setEditorPart(this);
        
        final IFile stringsFile = rootEditorInput.getFile().getParent().getFile( new Path( "strings.xml" ) ); //$NON-NLS-1$
        
        index = addPage( this.stringsSourceEditor, new FileEditorInput( stringsFile ) );
        setPageText( index, "Strings" ); //$NON-NLS-1$

    }
	
	@Override
	protected void setPartName(String partName) {
		super.setPartName("Android Resources"); //Looks like a hack, but no way to override part naming in other place unfortunately
	}
	
    @Override
    protected Element createModel() {
    	XmlEditorResourceStore foreignResStore = new XmlEditorResourceStore( this, this.stringsSourceEditor );
		foreignResource = new RootXmlResource( foreignResStore );
        RootXmlResource mainRes = createMainResource();
		this.androidResources = applicationElementType.instantiate( mainRes );
		foreignResource.init(androidResources);
        this.integratedAndroidResources = integratedAppElementType.instantiate( 
        		new RootIntegratedResource(mainRes, foreignResource) );
        
        return this.integratedAndroidResources;
    }

	protected RootXmlResource createMainResource() {
		RootXmlResource mainRes = new RootXmlResource( new XmlEditorResourceStore( this, this.colorsSourceEditor ) ) {
        	@Override
        	protected PropertyBinding createBinding(Property property) {
        		PropertyDef propertyDef = property.definition();
            	PropertyBinding binding = null;
            	if (propertyDef instanceof ListProperty) {
            		final IntegratedListBinding customBindingAnnotation = propertyDef.getAnnotation(IntegratedListBinding.class);
            		if (customBindingAnnotation != null) {
            			binding = new CustomizedXmlBindingImpl((XmlResource) foreignResource);
            			return binding;
            		} 
            	}
        		return super.createBinding(property);
        	}
        };
		return mainRes;
	}
    
    @Override
    protected void adaptModel( final Element model ) {
        final CorruptedResourceExceptionInterceptor interceptor 
            = new CorruptedResourceExceptionInterceptorImpl( getEditorSite().getShell() );
        
        this.androidResources.resource().setCorruptedResourceExceptionInterceptor( interceptor );
        foreignResource.setCorruptedResourceExceptionInterceptor( interceptor );
    }
    
    @Override
    protected final void createFormPages() throws PartInitException {
    	getModelElement();
    	
        Reference<EditorPageDef> ref = DefinitionLoader
	        .context( getClass().getClassLoader() )
	        .sdef( "com.genuitec.sapphire.integratededitor.ui.ResEditor" )
	        .page( "editor.page" );
        this.applicationDesignPage = new MasterDetailsEditorPage( this, this.androidResources, ref );
        addPage( 0, this.applicationDesignPage );
    }
    
    @Override
    public IContentOutlinePage getContentOutline( final Object page ) {
        if( page == this.colorsSourceEditor ) {
            return (IContentOutlinePage) this.colorsSourceEditor.getAdapter( IContentOutlinePage.class );
        }        
        return super.getContentOutline( page );
    }
    
}

